/*	do.cpp -- Digital Outputs (LED, buzzer, etc)
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#include <avr\io.h>
#include "abc.h"
#include "debug.h"
#include "do.h"
#include "timer.h"

char DO::m_fInit = fFalse;

/* NOTE: ENUM/LIST #defines are in iop.h */
const IOP DO::m_mpdopiop[dopMax] =
{
	DOP_LIST
};


DO::DO()
{
	if (!m_fInit)
		Init();
}

/* set direction for outputs; this uses DDRx sfr's [special function registers]
   and sets bits to 1 to indicate output */
void DO::Init() const
{
	const IOP *piop, *piopMax;

	for (piopMax = (piop = m_mpdopiop) + dopMax; piop < piopMax; ++piop)
		piop->SetAsOutputAndSignalHigh();

	m_fInit = fTrue;
}

/* note: 0 turns on LED; 1 turns it off */
void DO::On(uchar dop) const
{
	m_mpdopiop[dop].SignalLow();
}

void DO::Off(uchar dop) const
{
	m_mpdopiop[dop].SignalHigh();
}

/* blink c times, then pause */
void DO::Blink(uchar dop, uchar c) const
{
	const IOP *piop;
	TIMER timer;

	piop = &m_mpdopiop[dop];

	while (c-- > 0)
	{
		piop->SignalLow();	// LED on
		timer.WaitTicks(ctickSec / 4);
		piop->SignalHigh();	// LED off
		timer.WaitTicks(ctickSec / 4);
	}
//	timer.WaitTicks(ctickSec / 4);
}

