/*	eeprom.cpp
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#include <avr\io.h>
#include <avr\interrupt.h>	// for cli/sei
#include "abc.h"
#include "debug.h"
#include "eeprom.h"

void EEPROM::Write(ushort e, uchar b) const
{
	/* wait for EEPROM ready, then write */
	while (bit_is_set(EECR, EEWE))
		;
	/* FUTURE: wait for SPMEN in SPMCR to clear... (if do boot loader stuff) */
	EEAR = e;
	EEDR = b;
	/* turn off global interrupts during write (EEWE must be set
	   within 4 clock cycles of setting EEMWE) */
	cli();		// clear Global Interrupt Flag
	sbi(EECR, EEMWE);	// master write enable
	sbi(EECR, EEWE);	// write
	sei();
}

/* read the byte stored at EEPROM address e */
uchar EEPROM::Read(ushort e) const
{
	uchar b;

	while (bit_is_set(EECR, EEWE))
		;
	EEAR = (uint)e;
	sbi(EECR, EERE);
	b = EEDR;
	/* FUTURE: make sure Global Interrupt Flag is cleared during above?
	   [looks like not necessary?] */

	return b;
}

