/*	eyes.h
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#ifndef EYES_H
#define EYES_H

#include "ai.h"
#include "eeprom.h"

/* NOTE: keep in sync with m_ and s_ arrays in eyes.cpp */
enum	// Light Sensor
{
	lsFarLeft = 0,
	lsLeft,
	lsCenter,
	lsRight,
	lsFarRight,
	lsMax
};

enum	// index for Line Sensor Value array
{
	ilsvCur = 0,	// current reading
	ilsvOnLine,
	ilsvOffLine,
	ilsvMax
};

struct ELS	// Element: Light Sensor
{
	uchar aip;			// Analog Input Pin for this sensor
	ushort elsvBlack;	// EEPROM storage for black value
	ushort elsvWhite;	// EEPROM storage for white value
	uchar alsv[ilsvMax]; // sensor values for each position
};

class EYES
{
public:
	/* loads values for black/white level from EEPROM */
	EYES();

	/* read a light sensor (value stored in this struct, not returned) */
	void Read(uchar ls, const AI &ai) const;

	/* check for status of line under specified light sensor;
	   functions that take AI also take reading */
	char FLostLine(uchar ls) const;
	char FFoundLine(uchar ls) const;
	char FLostLine(uchar ls, const AI &ai) const;
	char FFoundLine(uchar ls, const AI &ai) const;

	/* return difference in sensor values (A - B)
	   (dlsv = Delta Line Sensor Value) */
	short Dlsv(uchar lsA, uchar lsB) const;

	/* read "black" level for all light sensors, storing
	   in EEPROM, and updating On/OffLine values */
	void MeterBlack(const AI &ai) const;

	/* read "white" level for all light sensors, storing
	   in EEPROM, and updating On/OffLine values */
	void MeterWhite(const AI &ai) const;

	static ELS m_dnls[lsMax];	// REVIEW: put back in private...
private:
	void SetLineValues() const;
	uchar LsvAvgReading(const AI &ai, uchar aip) const;

};

#endif // EEPROM_H
