/*	iop.h -- shared Input/Output stuff
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#ifndef IOP_H
#define IOP_H

class IOP	// Input/Output Pin/Port definition
{
public:
	/* REVIEW: could use nibbles if need more space... */
	uchar reg;		// register (from enum below)
	uchar bit;		// bit within register [port/pin Pxn, direction DDxn]

	/* stuff for inputs */
	void SetAsInput() const;
	void SetAsInputAndPullHigh() const;
	char FLow() const;	// true if signal is low (closed/signaled)

	/* stuff for outputs */
	void SetAsOutput() const;
	void SetAsOutputAndSignalHigh() const;
	void SignalHigh() const;	// (for output)
	void SignalLow() const;		// 
};

/* we can't use PORTx, PINx, or DDRx directly since they're memory
address lookups; use these to store in struct... */
enum
{
	regA = 0,	// PORTA, PINA, DDRA (Atmel "special function register")
	regB,
	regC,
	regD,
};

/*
*	PIN/PORT ASSIGNMENTS
*
*	These are used in di.cpp/h, ai.cpp/h, do.cpp/h, and mtr.cpp.h.
*	The enums and lists must be kept in sync!
*/

/*
*	Digital Input Pins
*
*	Btn = button
*/
/* REVIEW: actually put enums here?! */
#define DIP_ENUM	\
	dipBtnStart=0,	\
	dipBtnReset,	\
	dipBtnBlack,	\
	dipBtnWhite,	\
	dipMax
#define DIP_LIST	\
	{regC, 0},	/* button to start running (line follow) */		\
	{regC, 1},	/* button to reset [press & hold...] REVIEW */	\
	{regC, 6},	/* button to meter black */						\
	{regC, 7},	/* button to meter white */

/*
*	Analog Input Pins
*
*	Ls = line sensor
*/
#define AIP_ENUM	\
	aipLsFarLeft=0,	\
	aipLsLeft,		\
	aipLsCenter,	\
	aipLsRight,		\
	aipLsFarRight,	\
	aipMax
#define AIP_LIST	\
	{regA, 4},	/* far left light sensor (analog) */	\
	{regA, 1},	/* left light sensor (analog) */		\
	{regA, 2},	/* center light sensor (analog) */		\
	{regA, 3},	/* right light sensor (analog) */		\
	{regA, 5},	/* far right light sensor (analog) */

/*
*	Digital Output Ports
*
*	Led = LED
*	Bzr = buzzer
*/
#define DOP_ENUM	\
	dopLedGreen=0,	\
	dopMax
#define DOP_LIST	\
	{regB, 4},	/* green LED */

/*
*	MoToR Pins/ports
*
*	PWM = Pulse Width Modulation
*	Dir = direction
*	Ctr = rotation counter
*	Quad = quadrature input
*/
#define MTRP_ENUM	\
	mtrpPWMLeft=0,	\
	mtrpPWMRight,	\
	mtrpDirLeft,	\
	mtrpDirRight,	\
	mtrpOutMax,		\
	mtrpQuadTrigger = mtrpOutMax,	\
	mtrpQuadDir,	\
	mtrpMax,
#define MTRP_LIST	\
	{regD, 4},	/* left motor PWM */			\
	{regD, 5},	/* right motor PWM */			\
	{regC, 3},	/* left motor direction */		\
	{regC, 4},	/* right motor direction */		\
	{regD, 2},	/* right motor rotation count (INT0) (quadrature trigger) */ \
	{regC, 5},	/* right motor channel B (quadrature direction) */
#ifdef MAYBE	// REVIEW
	{regD, 3},	/* left motor rotation count (INT1) */ \
	{regC, 2},	/* left motor channel B */
#endif



#endif // IOP_H
