/*	mtr.h -- MoToR controls
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#ifndef MTR_H
#define MTR_H

#include "iop.h"

/* NOTE: ENUM/LIST #defines are in iop.h */
enum	// Motor Name
{
	MTRP_ENUM
};

class MTR
{
public:
	MTR();

	void Fwd(uchar spd) const;
	void Fwd(uchar spdLeft, uchar spdRight) const;
	void Rev(uchar spd) const;
	void Left(uchar spd) const;
	void Right(uchar spd) const;
	void Off() const;
	void On() const;	// restart, with last speed/direction
	void SetSpeed(uchar spd) const;	// keep same dir, change speed

	/* for measuring distance */
	void ClearDistance() const;
	short Distance() const;	// returns distance traveled since last clear
#ifdef OPTICAL_DIST
	void Count() const;		// called from interrupt routine
#endif

#ifdef NO
	void TestMotors() const;	// REVIEW: debug...
#endif

private:
	void Init() const;
	void Reset(uchar spd, uchar grfRev) const;
	void Reset(uchar spdLeft, uchar spdRight, uchar grfRev) const;

	static char m_fInit;
	static const IOP m_mpmtrpiop[mtrpMax];

	static uchar m_grfRev;		// flags indicate which motors in reverse

#ifdef OPTICAL_DIST
	static short m_cdu;	// du = Distance Unit; ~700 per 6", so 9800 for 7'
		/* short is enough for > 22' (pos or neg) */
#else // OPTICAL_DIST
	static ushort m_cdu;	// counting ticks; no direction...
#endif // OPTICAL_DIST
};

#endif // MTR_H
