/*	timer.cpp -- timer
*
*	Written by Cathy Saxton
*	cathy@idleloop.com
*/

#include <avr\io.h>
#include <avr\interrupt.h>
#include <avr\signal.h>
#include "abc.h"
#include "debug.h"
#include "timer.h"

/* this counts number of times we overflow the counter
   (every 256 (prescaled) clock pulses) */
static volatile ushort s_ctickOverflow = 0;
/* (this is volatile so the loops in Wait work!) */

char TIMER::m_fInit = fFalse;
//ushort TIMER::m_ctickMark = 0;

TIMER::TIMER()
{
	if (!m_fInit)
		Init();
}

void TIMER::Init() const
{
	m_fInit = fTrue;

	TCNT0 = 0;
#ifdef CLK_1MHZ
	TCCR0 = 0x03;		// 0000 00011 (prescaler: clk/64)
#else
	TCCR0 = 0x04;		// 0000 00100 (prescaler: clk/256)
#endif
	sbi(TIMSK, TOIE0);	// set Overflow Interrupt Enable (for timer/counter 0)

	/* REVIEW: should this be elsewhere? (e.g. main()) [also done in motors...] */
	/* set I-bit in SREG (global enable for interrupts) */
	sei();	// clear with cli();
}

void TIMER::Wait(uchar sec) const
{
	WaitTicks((ushort)sec * ctickSec);
}

void TIMER::WaitTicks(ushort ticks) const
{
	ushort c, cMax;

	c = s_ctickOverflow;
	cMax = c + ticks;

	if (cMax < c)
	{
		while (s_ctickOverflow > cMax)
			;	// go until wrap
	}
	while (s_ctickOverflow < cMax)
		;
}

/* returns current tick count */
ushort TIMER::CtickCur() const
{
	return s_ctickOverflow;
}

/* returns tick count for sec seconds from now */
ushort TIMER::CtickPlusSec(ushort sec) const
{
	return s_ctickOverflow + sec * ctickSec;
}

#ifdef UNUSED
/* sets vars with elapsed time since last mark;
   NULL OK for either/both param */
void TIMER::Mark(ulong *pctick, ulong *psec)
{
	ulong ctick, dtick;

	ctick = s_ctickOverflow;
	if (pctick != NULL || psec != NULL)
	{
		if (ctick > m_ctickMark)
		{
			dtick = ctick - m_ctickMark;
		}
		else
		{
			/* wrapped */
			dtick = 0xffffffff - m_ctickMark + 1 + ctick;
		}

		if (pctick != NULL)
			*pctick = dtick;	// REVIEW: wrong value... (sec is right)
		if (psec != NULL)
			*psec = (dtick + ctickSec/2) / ctickSec;
	}
	m_ctickMark = ctick;
}
#endif

INTERRUPT(SIG_OVERFLOW0)
{
	++s_ctickOverflow;
}
